package djmil.cordacheckers.cordaclient;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatThrownBy;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import djmil.cordacheckers.cordaclient.dao.GameState;
import djmil.cordacheckers.cordaclient.dao.GameState.Status;
import djmil.cordacheckers.cordaclient.dao.Piece;
import djmil.cordacheckers.user.HoldingIdentityResolver;

@SpringBootTest
public class GameBoardTests {
    @Autowired
    CordaClient cordaClient;

    @Autowired
    HoldingIdentityResolver holdingIdentityResolver;

    final String whitePlayerName = "alice";
    final String blackPlayerName = "bob";

    @Test
    void testSurrender() {
        final var hiWhite  = holdingIdentityResolver.getByUsername(whitePlayerName);
        final var hiBlack = holdingIdentityResolver.getByUsername(blackPlayerName);
        final String message = "GameBoard SURRENDER test";

        final GameState game = cordaClient.gameProposalCreate(
            hiWhite, hiBlack, Piece.Color.BLACK, message);

        System.out.println("Game UUID " +game.uuid());

        final GameState acceptedGameBlackView = cordaClient.gameProposalAccept(
            hiBlack, game.uuid());

        assertThat(acceptedGameBlackView.opponentColor()).isEqualByComparingTo(Piece.Color.WHITE);
        assertThat(acceptedGameBlackView.status()).isEqualByComparingTo(Status.GAME_BOARD_WAIT_FOR_OPPONENT);

        assertThatThrownBy(() -> { // Black can not surrender, since it is opponent's turn
            cordaClient.gameBoardSurrender(
                hiBlack, game.uuid());
        });

        final GameState surrendererGameView = cordaClient.gameBoardSurrender(
            hiWhite, game.uuid());

        assertThat(surrendererGameView.opponentName()).isEqualToIgnoringCase(blackPlayerName);
        assertThat(surrendererGameView.opponentColor()).isEqualByComparingTo(Piece.Color.BLACK);
        assertThat(surrendererGameView.status()).isEqualByComparingTo(Status.GAME_RESULT_YOU_LOOSE);

        final GameState winnerGameView = cordaClient.gameStateGet(
            hiBlack, game.uuid());

        assertThat(winnerGameView.opponentName()).isEqualToIgnoringCase(whitePlayerName);
        assertThat(winnerGameView.opponentColor()).isEqualByComparingTo(Piece.Color.WHITE);
        assertThat(winnerGameView.status()).isEqualByComparingTo(Status.GAME_RESULT_YOU_WON);
    }

    // @Test
    // void testGameBoardMove() throws JsonMappingException, JsonProcessingException, InvalidNameException {
    //     final var hiAlice = holdingIdentityResolver.getByUsername("alice");
    //     final var hiBob   = holdingIdentityResolver.getByUsername("bob");
    //     final var bobColor = Piece.Color.WHITE;

    //     final UUID gpUuid = cordaClient.gameProposalCreate(
    //         hiAlice, hiBob,
    //         bobColor, "GameBoard MOVE test"
    //     );

    //     System.out.println("New GameProposal UUID "+ gpUuid);

    //     final GameBoard gbState = cordaClient.gameProposalAccept(
    //         hiBob, gpUuid
    //     );

    //     System.out.println("New GameBoard UUID "+ gbState.gameUuid());

    //     assertThatThrownBy(() -> { // Alice can not move, since it is Bob's turn
    //         cordaClient.gameBoardMove(
    //             hiAlice, gbState.gameUuid(),
    //             Arrays.asList(1, 2));
    //     });

    //     final GameBoard gameBoard = cordaClient.gameBoardMove(
    //         hiBob, gbState.gameUuid(), Arrays.asList(1, 2));
    // }

    // private <T extends GameState> T findByUuid(List<T> statesList, UUID uuid) {
    //     for (T state : statesList) {
    //         if (state.uuid().compareTo(uuid) == 0)
    //             return state;
    //     };
    //     return null;
    // }
}
