package djmil.cordacheckers.contracts;

import static djmil.cordacheckers.contracts.GameCommand.requireThat;
import static djmil.cordacheckers.contracts.UtxoLedgerTransactionUtil.getSingleCommand;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.corda.v5.ledger.utxo.transaction.UtxoLedgerTransaction;

public class GameProposalContract implements net.corda.v5.ledger.utxo.Contract {

    private final static Logger log = LoggerFactory.getLogger(GameProposalContract.class);

    @Override
    public void verify(UtxoLedgerTransaction trx) {
        log.info("GameProposalContract.verify() called");

        requireThat(trx.getCommands().size() == 1, GameCommand.REQUIRE_SINGLE_COMMAND);
        final GameCommand command = getSingleCommand(trx, GameCommand.class);

        switch (command.action) {
            case GAME_PROPOSAL_CREATE:
                GameCommand.validateGameProposalCreate(trx);
                break;

            case GAME_PROPOSAL_ACCEPT:
                GameCommand.validateGameProposalAccept(trx);
                break;

            case GAME_PROPOSAL_REJECT:
                GameCommand.validateGameProposalReject(trx);
                break;

            case GAME_PROPOSAL_CANCEL:
                GameCommand.validateGameProposalCancel(trx);
                break;

            default:
                throw new GameCommand.ActionException();
        }
    }

}
