package djmil.cordacheckers.states;

import java.security.PublicKey;
import java.util.List;
import java.util.UUID;

import djmil.cordacheckers.contracts.GameProposalContract;
import net.corda.v5.base.annotations.ConstructorForDeserialization;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.BelongsToContract;

@BelongsToContract(GameProposalContract.class)
public class GameProposalState extends GameState {

    private final MemberX500Name issuer;
    private final MemberX500Name acquier;
    private final Piece.Color acquierColor;

    @ConstructorForDeserialization
    public GameProposalState(
        MemberX500Name issuer,
        MemberX500Name acquier,
        Piece.Color acquierColor,
        String message,
        UUID gameUuid,
        List<PublicKey> participants
    ) {
        super(gameUuid, message, participants);
        this.issuer = issuer;
        this.acquier = acquier;
        this.acquierColor = acquierColor;
    }

    public MemberX500Name getIssuer() {
        return issuer;
    }

    public MemberX500Name getAcquier() {
        return acquier;
    }

    public Piece.Color getAcquierColor() {
        return acquierColor;
    }

    public MemberX500Name getWhitePlayerName() {
        return acquierColor == Piece.Color.WHITE ? getAcquier() : getIssuer();
    }

    @Override
    public MemberX500Name getCounterpartyName(MemberX500Name myName) throws NotInvolved {
        if (issuer.compareTo(myName) == 0)
            return acquier;
    
        if (acquier.compareTo(myName) == 0)
            return issuer;
        
        throw new GameState.NotInvolved(myName, GameProposalState.class, this.gameUuid);
    }

}
