package djmil.cordacheckers.states;

import java.security.PublicKey;
import java.util.List;
import java.util.UUID;

import djmil.cordacheckers.contracts.GameResultContract;
import net.corda.v5.base.annotations.ConstructorForDeserialization;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.BelongsToContract;

@BelongsToContract(GameResultContract.class)
public class GameResultState extends GameState {

    private final MemberX500Name winnerName;
    private final MemberX500Name opponentName;

    private final Piece.Color winnerColor;
 
    @ConstructorForDeserialization
    public GameResultState(MemberX500Name winnerName, MemberX500Name opponentName, Piece.Color winnerColor, 
            UUID gameUuid, String message, List<PublicKey> participants) {
        super(gameUuid, message, participants);
        this.winnerName = winnerName;
        this.opponentName = opponentName;
        this.winnerColor = winnerColor;
    }

    public GameResultState(GameBoardState gameBoardState, MemberX500Name winnerName) {
        super(gameBoardState.gameUuid, null, gameBoardState.participants);
        this.opponentName = gameBoardState.getCounterpartyName(winnerName);
        this.winnerName = winnerName;
        this.winnerColor = gameBoardState.getWhitePlayerName().compareTo(winnerName) == 0 ? Piece.Color.WHITE : Piece.Color.BLACK;
    }

    public MemberX500Name getWinnerName() {
        return winnerName;
    }

    public MemberX500Name getOpponentName() {
        return opponentName;
    }

    public Piece.Color getWinnerColor() {
        return winnerColor;
    }

    @Override
    public MemberX500Name getCounterpartyName(MemberX500Name myName) throws NotInvolved {
        if (winnerName.compareTo(myName) == 0)
            return opponentName;
    
        if (opponentName.compareTo(myName) == 0)
            return winnerName;
        
        throw new GameState.NotInvolved(myName, GameResultState.class, this.gameUuid);
    }

    @Override
    public MemberX500Name getWhitePlayerName() {
        return winnerColor == Piece.Color.WHITE ? winnerName : opponentName;
    }

}
