package djmil.cordacheckers.states;

import java.security.PublicKey;
import java.util.List;
import java.util.UUID;

import org.jetbrains.annotations.NotNull;

import net.corda.v5.base.annotations.CordaSerializable;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.ContractState;

@CordaSerializable
public abstract class GameState implements ContractState {
    final UUID gameUuid;
    final String message;
    final List<PublicKey> participants;

    @NotNull
    public abstract MemberX500Name getWhitePlayerName();

    @NotNull
    public abstract MemberX500Name getCounterpartyName(MemberX500Name myName) throws NotInvolved;

    GameState(UUID gameUuid, String message, List<PublicKey> participants) {
        this.gameUuid = gameUuid;
        this.message = message;
        this.participants = participants;
    }

    public UUID getGameUuid() {
        return gameUuid;
    }

    public String getMessage() {
        return message;
    }

    public List<PublicKey> getParticipants() {
        return participants;
    }

    public MemberX500Name getBlackPlayerName() {
        return getCounterpartyName(getWhitePlayerName());
    }

    public Piece.Color getCounterpartyColor(MemberX500Name myName) throws NotInvolved {
        final MemberX500Name opponentName = getCounterpartyName(myName);

        return getWhitePlayerName().compareTo(opponentName) == 0 ? Piece.Color.WHITE : Piece.Color.BLACK;
    }

    public static class NotInvolved extends RuntimeException {
        public <T> NotInvolved(MemberX500Name name, Class<T> clazz, UUID uuid) {
            super(name +" is not involved in " +clazz.getSimpleName() +" UUID " +uuid);
        }
    }

}
