package djmil.cordacheckers.states;

import net.corda.v5.base.annotations.ConstructorForDeserialization;
import net.corda.v5.base.annotations.CordaSerializable;

@CordaSerializable
public class Piece {
    
    @CordaSerializable
    public enum Type {
        MAN,
        KING;
    }

    @CordaSerializable
    public enum Color {
        WHITE,
        BLACK;

        public static Color oppositOf(Color color) {
            switch (color) {
                case WHITE:
                    return BLACK;
                case BLACK:
                    return WHITE;
                default:
                    throw new UnknownException();
            }
        }

        public static class UnknownException extends RuntimeException {
            public UnknownException() {
                super("Unknown Color value");
            }
        }
    }

    private final Color color;
    private final Type type;

    @ConstructorForDeserialization
    public Piece(Color color, Type type) {
        this.color = color;
        this.type = type;
    }

    public Color getColor() {
        return color;
    }

    public Type getType() {
        return type;
    }

}
