package djmil.cordacheckers.gameproposal;

import java.time.Duration;
import java.time.Instant;
import java.util.UUID;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.contracts.GameCommand;
import djmil.cordacheckers.gamestate.CommitSubFlow;
import djmil.cordacheckers.gamestate.FlowResponce;
import djmil.cordacheckers.gamestate.GetFlow;
import djmil.cordacheckers.gamestate.View;
import djmil.cordacheckers.gamestate.ViewBuilder;
import djmil.cordacheckers.states.GameState;
import net.corda.v5.application.flows.ClientRequestBody;
import net.corda.v5.application.flows.ClientStartableFlow;
import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.flows.FlowEngine;
import net.corda.v5.application.marshalling.JsonMarshallingService;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.crypto.SecureHash;
import net.corda.v5.ledger.utxo.StateAndRef;
import net.corda.v5.ledger.utxo.UtxoLedgerService;
import net.corda.v5.ledger.utxo.transaction.UtxoSignedTransaction;

public class CancelFlow implements ClientStartableFlow{

     private final static Logger log = LoggerFactory.getLogger(CancelFlow.class);

     @CordaInject
     public JsonMarshallingService jsonMarshallingService;

     @CordaInject
     public UtxoLedgerService utxoLedgerService;

     @CordaInject
     public FlowEngine flowEngine;

     @Suspendable
     @Override
     public String call(ClientRequestBody requestBody) {
          SecureHash gameStateUtxoTrxId = null;

          try {
               final GameCommand command = new GameCommand(GameCommand.Action.GAME_PROPOSAL_CANCEL);

               final UUID gameStateUuid = UUID.fromString(requestBody.getRequestBody());

               final StateAndRef<GameState> inputState = this.flowEngine
                    .subFlow(new GetFlow(gameStateUuid));

               final UtxoSignedTransaction gameStateRejectTrx = utxoLedgerService.createTransactionBuilder()
                    .addCommand(command)
                    .addInputState(inputState.getRef())
                    .addSignatories(inputState.getState().getContractState().getParticipants())
                    .setNotary(inputState.getState().getNotaryName())
                    .setTimeWindowUntil(Instant.now().plusMillis(Duration.ofDays(1).toMillis()))
                    .toSignedTransaction();

               gameStateUtxoTrxId = this.flowEngine
                    .subFlow(new CommitSubFlow(gameStateRejectTrx, command.getObserver(inputState)));

               final View gameStateView = this.flowEngine
                    .subFlow(new ViewBuilder(gameStateUtxoTrxId));

               return new FlowResponce(gameStateView, gameStateUtxoTrxId)
                    .toJsonEncodedString(jsonMarshallingService);
          } 
          catch (Exception e) {
               log.warn("Exception during processing " + requestBody + " request: " + e.getMessage());
               return new FlowResponce(e, gameStateUtxoTrxId)
                    .toJsonEncodedString(jsonMarshallingService);
          }
     }

}
