package djmil.cordacheckers.gamestate;

import java.util.List;
import java.util.Map;
import java.util.UUID;

import djmil.cordacheckers.states.GameBoardState;
import djmil.cordacheckers.states.GameProposalState;
import djmil.cordacheckers.states.GameResultState;
import djmil.cordacheckers.states.Piece;
import net.corda.v5.base.types.MemberX500Name;


// GameBoard from the player's point of view
public class View {
    public static enum Status {
        GAME_PROPOSAL_WAIT_FOR_OPPONENT,
        GAME_PROPOSAL_WAIT_FOR_YOU,
        GAME_PROPOSAL_REJECTED,
        GAME_PROPOSAL_CANCELED,
        
        GAME_BOARD_WAIT_FOR_OPPONENT,
        GAME_BOARD_WAIT_FOR_YOU,

        GAME_RESULT_YOU_WON,
        GAME_RESULT_YOU_LOOSE;
    }
    public final Status status;

    public final String opponentName;
    public final Piece.Color opponentColor;

    public final Map<Integer, Piece> board;
    public final Integer moveNumber;
    public final List<Integer> previousMove;

    public final String message;
    public final UUID uuid;

    // Serialisation service requires a default constructor
    public View() {
        this.status = null;
        this.opponentName = null;
        this.opponentColor = null;
        this.board = null;
        this.moveNumber = null;
        this.previousMove = null;
        this.message = null;
        this.uuid = null;
    }

    public View(View.Status status, GameProposalState gameProposal, MemberX500Name myName) {
        this.status = status;
        this.opponentName = gameProposal.getCounterpartyName(myName).getCommonName();
        this.opponentColor = gameProposal.getCounterpartyColor(myName);
        this.board = null;
        this.moveNumber = null;
        this.previousMove = null;
        this.message = gameProposal.getMessage();
        this.uuid = gameProposal.getGameUuid();
    }

    public View(View.Status status, GameBoardState gameBoard, MemberX500Name myName) {
        this.status = status;
        this.opponentName = gameBoard.getCounterpartyName(myName).getCommonName();
        this.opponentColor = gameBoard.getCounterpartyColor(myName);
        this.board = gameBoard.getBoard();
        this.moveNumber = gameBoard.getMoveNumber();
        this.previousMove = null;
        this.message = gameBoard.getMessage();
        this.uuid = gameBoard.getGameUuid();
    }

    public View(View.Status status, GameBoardState gameBoard, List<Integer> previousMove, MemberX500Name myName) {
        this.status = status;
        this.opponentName = gameBoard.getCounterpartyName(myName).getCommonName();
        this.opponentColor = gameBoard.getCounterpartyColor(myName);
        this.board = gameBoard.getBoard();
        this.moveNumber = gameBoard.getMoveNumber();
        this.previousMove = previousMove;
        this.message = gameBoard.getMessage();
        this.uuid = gameBoard.getGameUuid();
    }

    public View(View.Status status, GameResultState gameResult, MemberX500Name myName) {
        this.status = status;
        this.opponentName = gameResult.getCounterpartyName(myName).getCommonName();
        this.opponentColor = gameResult.getCounterpartyColor(myName);
        this.board = null;
        this.moveNumber = null;
        this.previousMove = null;
        this.message = gameResult.getMessage();
        this.uuid = gameResult.getGameUuid();
    }

}
